from typing import Dict, Set
from urllib.parse import urljoin
from airflow.secrets import BaseSecretsBackend
import requests
from datacat.DataCatConnection import DataCatConnection

from hooks.DataCatalogHook import DataCatalogHook, get_connection_from_entry, get_connection_with_secrets_from_entry

connection_backend_type = "airflow_connections"


class DataCatConnectionWithSecrets(DataCatConnection):
    def __init__(self, catalog_url: str = "", username: str = "", password: str = ""):
        super().__init__(catalog_url=catalog_url, username=username, password=password)

    def get_secrets_keys(self, datacat_type: str, oid: str) -> Set:
        headers = {
            'accept' : 'application/json',
            'Content-Type' : 'application/json',
            'Authorization' : 'Bearer {}'.format(self._auth_token)
        }
        url = urljoin(self.url, f"{datacat_type}/{oid}/secrets")
        response = requests.get(url, headers=headers)
        if response.ok:
            return response.json()
        else:
            raise ConnectionError(response.text)

    def get_all_secret_key_value(self, datacat_type: str, oid: str) -> Dict[str,str]:
        headers = {
            'accept' : 'application/json',
            'Content-Type' : 'application/json',
            'Authorization' : 'Bearer {}'.format(self._auth_token)
        }
        url = urljoin(self.url, f"{datacat_type}/{oid}/secrets_values")
        response = requests.get(url, headers=headers)
        if response.ok:
            return response.json()
        else:
            raise ConnectionError(response.text)

    def get_secret_value(self, datacat_type: str, oid: str, key: str) -> str:
        headers = {
            'accept' : 'application/json',
            'Content-Type' : 'application/json',
            'Authorization' : 'Bearer {}'.format(self._auth_token)
        }
        url = urljoin(self.url, f"{datacat_type}/{oid}/secrets/{key}")
        response = requests.get(url, headers=headers)
        if response.ok:
            return response.json()
        else:
            raise ConnectionError(response.text)

class DatacatSecretsBackend(BaseSecretsBackend):
    """A Backend for directly managing connections in apache airflow"""

    # TODO init with datacat elevated login to prevent secrets acces from jobs

    def get_connection(self, conn_id: str):
        """returns a Connection object created from the <conenction_type>/<conn_id> object in the datacatalog"""
        # only for testing: check that a specific oid has been requested
        if conn_id != "860355e9-975f-4253-9421-1815e20c879b":
            return None

        
        hook = DataCatalogHook("datacatalog")
        secrets_conn = DataCatConnectionWithSecrets(hook.connection.url, hook.connection.user, hook.connection._password)
        data = secrets_conn.get_object(connection_backend_type, conn_id)
        secrets = secrets_conn.get_all_secret_key_value(connection_backend_type, conn_id)
        conn = get_connection_from_entry(data, secrets, connection_backend_type, conn_id)
        return conn